#!/bin/bash

### The simulation part of the neural network. This is run by individual chunks in parallel with each other

### 1. Creates directories for each member if they havent already done so, i.e. in the first generation
### 2. Edits the namelists of each member using the corresponding row of the 00.csv parameters file
### 3. Runs the simulation
### 4. Uses the outputs, reference bias and observed data to collect the cost and save as a .txt

## ALL SHOULD BE THE SAME AS THOSE IN pre.sh
SCRHOME=/gpfs/scratch/bsc32/bsc32786/
MODEL=${SCRHOME}%EXPID%/EXP00/
OBS=${SCRHOME}pisces1d/ORCA2_OFF_PISCES/obs/Mprof_6901486_L31_month_2015-01-01_2015-12-31.nc
MODELFILE=PISCES1D_1m_19600101_19691231_ptrc_T.nc


### 1. Creates directories for each member if they havent already done so, i.e. in the first generation

shopt -s nullglob
numfiles=(${MODEL}parameters/*)
numfiles=${#numfiles[@]}

INPUTS=input/
XIOS=xios-2.5/
NEMO=release-4.0/

chunk=%CHUNK%
printf -v chunk "%04d" $chunk

if [ $numfiles -eq $((1)) ]; then
    # MAKE EXPID DIRECTORY AND COPY ALL THE CONTENT FROM EXP00 (.XML, EXECUTABLES, AND NAMELISTS)
    cd ${MODEL}
    mkdir -p $chunk
    cd $chunk
    cp -R ${MODEL}default/nemo .
    cp -R ${MODEL}default/namelist* .
    cp -R ${MODEL}default/*.xml .
    cp -R ${MODEL}cropped/* .
fi


### 2. Edits the namelists of each member using the corresponding row of the 0000.csv parameters file

VARIABLES=variables.csv
NAMELIST=namelist_pisces_ref

cd ${MODEL}$chunk/

module load python

function changenls {

nls="$(python - $1 $2 $3 $4 << END
# inputs:
# 1. Parameters file (0000.csv)
# 2. Variables file (variables.csv)
# 3. Namelist (namelist_pisces_ref)
# 4. Chunk number

import os
import pandas as pd
import numpy as np

values=np.genfromtxt('$1')
names=pd.read_csv('$2')

chunk=int('$4')-1

with open('$3','r') as nl:
    with open('temp','w+') as t:
        lines=nl.readlines()
        for l in lines:
            for j in range(len(values[chunk])):
                if names['var'][j]+' ' in l and "=" in l:
                    l = '   '+ names['var'][j] + '  =  ' + str(values[chunk][j]) + '\n'
            t.write(l)

os.remove('$3')
os.rename('temp','$3')

END
)"
}

changenls ${MODEL}parameters/0000.csv ${MODEL}$VARIABLES ${MODEL}$chunk/$NAMELIST %CHUNK%


### 3. Runs the simulation

cd ${MODEL}$chunk

module load intel/2017.4
module load impi/2017.4
module load netcdf/4.2
module load hdf5/1.8.19
module load perl

mpirun -np 1 ./nemo

### 4. Uses the outputs, reference bias and observed data to collect the cost and save as a .txt

module load netcdf/4.4.1.1

function getcost {

cost="$(python - $1 $2 $3 $4 << END
# $1 - path to reference bias, $2 - path to observed data $3 - Path to output

from netCDF4 import Dataset
import numpy as np
import os
import math

# get the reference bias

os.chdir('$1')
bbp700refbias=np.genfromtxt('bbp700refbias.csv')
chlarefbias=np.genfromtxt('chlarefbias.csv')

# get the observed data

obs=Dataset('$2')
bbp700obs = np.array(obs.variables['bbp700_despiked'])
bbp700obs = np.reshape(bbp700obs, (np.shape(bbp700obs)[0], np.shape(bbp700obs)[1]))
bbp700obs[bbp700obs == 0.0] = np.nan
chlaobs = np.array(obs.variables['chla_cal'])
chlaobs = np.reshape(chlaobs, (np.shape(chlaobs)[0], np.shape(chlaobs)[1]))
chlaobs[chlaobs == 0.0] = np.nan
depths=np.array(obs.variables['deptht'])
obs.close()

# get the model data
os.chdir('$3')
mod = Dataset('$4')
bbp700mod = (np.array(mod.variables['POC'])[-60:, :, 1, 1]+np.array(mod.variables['PHY'])[-60:, :, 1, 1])*0.001
chlamod = (np.array(mod.variables['DCHL'])[-60:, :, 1, 1]+np.array(mod.variables['NCHL'])[-60:, :, 1, 1])
for i in range(12):
    bbp700mod[i] = np.mean(bbp700mod[i::12],0)
    chlamod[i] = np.mean(chlamod[i::12],0)
bbp700mod = bbp700mod[:12]
chlamod = chlamod[:12]
bbp700mod[bbp700mod == 200000039192547557376] = np.nan
chlamod[chlamod == 200000039192547557376] = np.nan
bbp700mod=bbp700mod[:,:np.shape(bbp700obs)[1]]
chlamod=chlamod[:,:np.shape(chlaobs)[1]]
mod.close()

# converting the data into logspace and normalising based on observed data

bbp700obs=np.log10(bbp700obs)
bbp700mod=np.log10(bbp700mod)
chlaobs=np.log10(chlaobs)
chlamod=np.log10(chlamod)

def gradperiod(A,d):
    A=np.concatenate(([A[-1]],A,[A[0]]),axis=0)
    B=np.gradient(A,1,d)
    return B[0][1:-1]#+B[0][1:-1]

#bbp700obs=gradperiod(bbp700obs,depths)[:,10:]
#bbp700mod=gradperiod(bbp700mod,depths)[:,10:]
#chlaobs=gradperiod(chlaobs,depths)[:,:17]
#chlamod=gradperiod(chlamod,depths)[:,:17]

bbp700obs=bbp700obs[:,10:]
bbp700mod=bbp700mod[:,10:]
chlaobs=chlaobs[:,:17]
chlamod=chlamod[:,:17]

# calculating the statistical parameters

names=['bias','normstd','R','RMSE','cost']

bias=np.nanmean(bbp700mod-bbp700obs)
mbias=bias/bbp700refbias
mstd=np.nanstd(bbp700mod)/np.nanstd(bbp700obs)
R=float(np.ma.corrcoef(np.ma.masked_invalid(bbp700obs.flatten()), np.ma.masked_invalid(bbp700mod.flatten()))[0,1])

#S3=1.-(math.exp(-((mstd-1.)**2)/0.18))*((1+R)/2) # S3
#S3=1.-(((1.+R)*2)/(mstd**2+mstd**(-2)+2)) #S1
S3=1.-(((1.+R)**4.)/(4.*(mstd**2+mstd**(-2)+2))) #S2

RMSE=math.sqrt(np.nansum((bbp700obs-bbp700mod)**2)/bbp700obs.size)
ST=math.sqrt((mbias**2)+(S3**2))

stats=[bias,mstd,R,RMSE,ST]

for i in range(len(names)):
    np.savetxt('bbp700'+names[i]+'.csv',[stats[i]])

bias=np.nanmean(chlamod-chlaobs)
mbias=bias/chlarefbias
mstd=np.nanstd(chlamod)/np.nanstd(chlaobs)
R=float(np.ma.corrcoef(np.ma.masked_invalid(chlaobs.flatten()), np.ma.masked_invalid(chlamod.flatten()))[0,1])
S3=1.-(math.exp(-((mstd-1.)**2)/0.18))*((1+R)/2)
RMSE=math.sqrt(np.nansum((chlaobs-chlamod)**2)/chlaobs.size)
ST=math.sqrt((mbias**2)+(S3**2))

stats=[bias,mstd,R,RMSE,ST]

for i in range(len(names)):
    np.savetxt('chla'+names[i]+'.csv',[stats[i]])

END
)"
}

getcost ${MODEL}default/ $OBS $MODEL${chunk} $MODELFILE
